//-----------------------------------------------------------------------------
// name: Game.cpp
//-----------------------------------------------------------------------------
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>

#include "stgl.h"
#include "st.h"

#include <SDL/SDL.h>

#include "shatter.h"
#include "bufferfish.h"
#include "game.h"
#include "ticktimer.h"
#include "textfile.h"

using namespace std;

// some defines

#ifdef __LINUX_COLEMAK__
#define UP_KEY SDLK_w
#define DOWN_KEY SDLK_r
#define LEFT_KEY SDLK_a
#define RIGHT_KEY SDLK_s
#else
#define UP_KEY SDLK_w
#define DOWN_KEY SDLK_s
#define LEFT_KEY SDLK_a
#define RIGHT_KEY SDLK_d
#endif

bool hack = false;

Game & Game::get_singleton() {
    static Game singleton;
    hack = true;
    return singleton;
}

// entry point
Game::Game()
{
    _window_width = 800;
    _window_height = 600;
    _is_mouse_pressed = false;

    init_sdl();
    init_gl();
    init_audio();

    init_scene();
}

Game::~Game() {
}

double r_dbl() {
    return 2.0 * rand()/RAND_MAX - 1;
}
// *** init functions *** //
void Game::init_scene() {
    _death_ball = new DeathBall(_sound_sys);
    _camera = new Camera(_death_ball);

//  for (int k = 0; k < 10; k++) {
//      GravityBall* gball = new GravityBall(_sound_sys, "content/audio/Contemplative Synth 02.wav", 2);
//      gball->_pos.x = k*2;
//      _gravity_balls.push_back(gball);
//
//      for (int k = 0; k < 5; k++) {
//          BufferFish* fish = new BufferFish(_sound_sys, "content/audio/bwoop.wav", _death_ball);
//          fish->_gravity_ball = gball;
//          //fish->_pos.x = k*8;
//
//          _buffer_fishes.push_back(fish);
//      }
//  }

    /* level design is hardcoded in below */

    GravityBall* gball;
    BufferFish* fish;

    gball = new GravityBall(_sound_sys, "content/audio/borealis.wav", _death_ball, 10);
    gball->_pos = STVector3(3, 0, 5);
    _gravity_balls.push_back(gball);

    gball = new GravityBall(_sound_sys, "content/audio/borealis.wav", _death_ball, 10);
    gball->_pos = STVector3(-3, 0, 5);
    _gravity_balls.push_back(gball);

    gball = new GravityBall(_sound_sys, "content/audio/welcome.wav", _death_ball, 5);
    gball->_pos = STVector3(0, 0, 5);
    _gravity_balls.push_back(gball);

    gball = new GravityBall(_sound_sys, "content/audio/borealis.wav", _death_ball, 10);
    gball->_pos = STVector3(3, 0, 15);
    _gravity_balls.push_back(gball);

    gball = new GravityBall(_sound_sys, "content/audio/borealis.wav", _death_ball, 10);
    gball->_pos = STVector3(-3, 0, 15);
    _gravity_balls.push_back(gball);

    for (unsigned k = 0; k < 8; k++) {
        gball = new GravityBall(_sound_sys, "content/audio/Contemplative Synth 02.wav", _death_ball, 2);
        gball->_pos = STVector3(.2*k*k, 0, 50+k*2);
        _gravity_balls.push_back(gball);
        BufferFish* fish = new BufferFish(_sound_sys, "content/audio/bwoop.wav", _death_ball);
        fish->_gravity_ball = gball;
        fish->_pos = STVector3(.2*k*k+2, 1, 50+k*2);
        fish->_pos = STVector3(.2*k*k+2, -1, 50+k*2);
        _buffer_fishes.push_back(fish);
    }

    gball = new GravityBall(_sound_sys, "content/audio/electric_piano.wav", _death_ball, 6);
    gball->_pos = STVector3(35, 0, 60);
    _gravity_balls.push_back(gball);
    for (unsigned k = 0; k < 6; k++) {
        BufferFish* fish = new BufferFish(_sound_sys, "content/audio/bwoop.wav", _death_ball);
        fish->_gravity_ball = gball;
        fish->_pos = STVector3(35+ r_dbl(), 0 + r_dbl(), 60 + r_dbl());
        _buffer_fishes.push_back(fish);
    }

    gball = new GravityBall(_sound_sys, "content/audio/electric_piano.wav", _death_ball, 6);
    gball->_pos = STVector3(35, 0, 66);
    _gravity_balls.push_back(gball);
    for (unsigned k = 0; k < 6; k++) {
        BufferFish* fish = new BufferFish(_sound_sys, "content/audio/bwoop.wav", _death_ball);
        fish->_gravity_ball = gball;
        fish->_pos = STVector3(35 + r_dbl(), 0 + r_dbl(), 66 + r_dbl());
        _buffer_fishes.push_back(fish);
    }

    gball = new GravityBall(_sound_sys, "content/audio/electric_piano.wav", _death_ball, 6);
    gball->_pos = STVector3(41, 0, 66);
    _gravity_balls.push_back(gball);
    for (unsigned k = 0; k < 6; k++) {
        BufferFish* fish = new BufferFish(_sound_sys, "content/audio/bwoop.wav", _death_ball);
        fish->_gravity_ball = gball;
        fish->_pos = STVector3(41+ r_dbl(), 0 + r_dbl(), 66 + r_dbl());
        _buffer_fishes.push_back(fish);
    }

    gball = new GravityBall(_sound_sys, "content/audio/electric_piano.wav", _death_ball, 6);
    gball->_pos = STVector3(41, 0, 60);
    _gravity_balls.push_back(gball);
    for (unsigned k = 0; k < 6; k++) {
        BufferFish* fish = new BufferFish(_sound_sys, "content/audio/bwoop.wav", _death_ball);
        fish->_gravity_ball = gball;
        fish->_pos = STVector3(41+ r_dbl(), 0 + r_dbl(), 60 + r_dbl());
        _buffer_fishes.push_back(fish);
    }

    gball = new GravityBall(_sound_sys, "content/audio/Deep Pad 01.wav", _death_ball, 30);
    gball->_pos = STVector3(60, 0, 64);
    _gravity_balls.push_back(gball);
    for (unsigned k = 0; k < 8; k++) {
        BufferFish* fish = new BufferFish(_sound_sys, "content/audio/Deep Pad 03.wav", _death_ball);
        fish->_gravity_ball = gball;
        fish->_pos = STVector3(60+ 2*r_dbl(), 0 + 2*r_dbl(), 64 + 2*r_dbl());
        _buffer_fishes.push_back(fish);
    }

    gball = new GravityBall(_sound_sys, "content/audio/Deep Pad 01.wav", _death_ball, 30);
    gball->_pos = STVector3(60, 0, 60);
    _gravity_balls.push_back(gball);
    for (unsigned k = 0; k < 8; k++) {
        BufferFish* fish = new BufferFish(_sound_sys, "content/audio/Deep Pad 03.wav", _death_ball);
        fish->_gravity_ball = gball;
        fish->_pos = STVector3(60+ 2*r_dbl(), 0 + 2*r_dbl(), 60 + 2*r_dbl());
        _buffer_fishes.push_back(fish);
    }

    for (unsigned k = 0; k < 40; k++) {
        gball = new GravityBall(_sound_sys, "content/audio/clicky.wav", _death_ball, 1);
        gball->_pos = STVector3(80 + 10 * r_dbl(), 0, 60 + 5 * r_dbl());
        _gravity_balls.push_back(gball);
    }

    gball = new GravityBall(_sound_sys, "content/audio/Deep Pad 01.wav", _death_ball, 30);
    gball->_pos = STVector3(130, 0, 64);
    _gravity_balls.push_back(gball);
    for (unsigned k = 0; k < 8; k++) {
        BufferFish* fish = new BufferFish(_sound_sys, "content/audio/Deep Pad 03.wav", _death_ball);
        fish->_gravity_ball = gball;
        fish->_pos = STVector3(130+ 2*r_dbl(), 0 + 2*r_dbl(), 64 + 2*r_dbl());
        _buffer_fishes.push_back(fish);
    }

    gball = new GravityBall(_sound_sys, "content/audio/Deep Pad 01.wav", _death_ball, 30);
    gball->_pos = STVector3(130, 0, 60);
    _gravity_balls.push_back(gball);
    for (unsigned k = 0; k < 8; k++) {
        BufferFish* fish = new BufferFish(_sound_sys, "content/audio/Deep Pad 03.wav", _death_ball);
        fish->_gravity_ball = gball;
        fish->_pos = STVector3(130+ 2*r_dbl(), 0 + 2*r_dbl(), 60 + 2*r_dbl());
        _buffer_fishes.push_back(fish);
    }

}

int Game::init_audio() {
    unsigned int            version;

    /*
        Create a System object and initialize.
    */
    FMOD_RESULT result = FMOD::System_Create(&_sound_sys);
    FMODERRCHECK(result);

    result = _sound_sys->getVersion(&version);
    FMODERRCHECK(result);

    if (version < FMOD_VERSION)
    {
        printf("Error!  You are using an old version of FMOD %08x.  This program requires %08x\n", version, FMOD_VERSION);
        return 0;
    }

    int n_channels = 256;
    result = _sound_sys->setSoftwareChannels(n_channels);
    FMODERRCHECK(result);
    result = _sound_sys->init(n_channels, FMOD_INIT_NORMAL, 0);
    FMODERRCHECK(result);

    cout << "audio initialization is a success!" << endl;
    return 0;
}

/* I took this code from an example at http://www.opengl.org/sdk/docs/tutorials/ClockworkCoders/lighting.php */
void DemoLight(void)
{
     glEnable(GL_LIGHTING);
     glEnable(GL_LIGHT0);
     glEnable(GL_LIGHT1);
     glEnable(GL_NORMALIZE);
     
     // Light model parameters:
     // -------------------------------------------
     
     GLfloat lmKa[] = {0.0, 0.0, 0.0, 0.0 };
     glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmKa);
     
     glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER, 1.0);
     glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, 0.0);
     
     // -------------------------------------------
     // Spotlight Attenuation
     
//   GLfloat spot_direction[] = {1.0, -1.0, -1.0 };
//   GLint spot_exponent = 30;
//   GLint spot_cutoff = 180;
//   
//   glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, spot_direction);
//   glLighti(GL_LIGHT0, GL_SPOT_EXPONENT, spot_exponent);
//   glLighti(GL_LIGHT0, GL_SPOT_CUTOFF, spot_cutoff);
    
     GLfloat Kc = 0.1;
     GLfloat Kl = 0.5;
     GLfloat Kq = 0.05;
     
     glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION,Kc);
     glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, Kl);
     glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, Kq);

     Kc = 1;
     Kl = 0;
     Kq = 0;

     glLightf(GL_LIGHT1, GL_CONSTANT_ATTENUATION,Kc);
     glLightf(GL_LIGHT1, GL_LINEAR_ATTENUATION, Kl);
     glLightf(GL_LIGHT1, GL_QUADRATIC_ATTENUATION, Kq);
     
     
     // ------------------------------------------- 
     // Lighting parameters:

     {
     GLfloat light_pos[] = {0.0f, 5.0f, 5.0f, 1.0f};
     GLfloat light_Ka[]  = {0.1f, 0.1f, 0.05f, 1.0f};
     GLfloat light_Kd[]  = {1.0f, 1.0f, 1.0f, 1.0f};
     GLfloat light_Ks[]  = {1.0f, 1.0f, 1.0f, 1.0f};

     glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
     glLightfv(GL_LIGHT0, GL_AMBIENT, light_Ka);
     glLightfv(GL_LIGHT0, GL_DIFFUSE, light_Kd);
     glLightfv(GL_LIGHT0, GL_SPECULAR, light_Ks);
     }

     {
     GLfloat light_pos[] = {0.0f, -20.0f, 10.0f, 1.0f};
     GLfloat light_Ka[]  = {0.4f, 0.4f, 0.4f, 1.0f};
     GLfloat light_Kd[]  = {0.5f, 0.5f, 0.5f, 1.0f};
     GLfloat light_Ks[]  = {0.5f, 0.5f, 0.5f, 1.0f};

     glLightfv(GL_LIGHT1, GL_POSITION, light_pos);
     glLightfv(GL_LIGHT1, GL_AMBIENT, light_Ka);
     glLightfv(GL_LIGHT1, GL_DIFFUSE, light_Kd);
     glLightfv(GL_LIGHT1, GL_SPECULAR, light_Ks);
     }

     // -------------------------------------------
     // Material parameters:

}

void Game::init_gl() {
    glEnable(GL_BLEND);
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    DemoLight();

    _shader = new STShaderProgram;
    _shader->LoadVertexShader("content/shaders/vertexshader.txt");
    _shader->LoadFragmentShader("content/shaders/fragmentshader.txt");
    _shader->Bind();
//    glEnable(GL_TEXTURE_2D);
}

/* initialize sdl screen*/
int Game::init_sdl() {

    SDL_Surface *screen;

    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER) < 0) {
        fprintf(stderr,
	        "Unable to initialize SDL: %s\n",
	        SDL_GetError());
        return(-1);
    }

    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
    SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8);
    SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8);
    SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8);
    SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, 8);

    putenv(strdup("SDL_VIDEO_CENTERED=center"));

    if (!(screen = SDL_SetVideoMode(_window_width, _window_height,
				    0, SDL_SWSURFACE | SDL_RESIZABLE | SDL_OPENGL))) {
        fprintf(stderr,
	        "Unable to set video mode: %s\n",
	        SDL_GetError());
        return(-1);
    }

    SDL_WM_SetCaption("Realtime Game", NULL);

    // hide the mouse and keep the window
    //SDL_WM_GrabInput(SDL_GRAB_ON);
    SDL_WarpMouse(_window_width/2, _window_height/2);
    SDL_ShowCursor(false);

    do_gl_projection();

    return(0);                
}

void Game::clean_up() {
    // stub remove?
}


void Game::do_gl_projection() {
    double aspect = double(_window_width)/_window_height;
    // set up coordinate space 
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();

    gluPerspective(45, -aspect, .1, 10000);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    gluLookAt (0,0,0,
               0,0,1,
               0,1,0);
}
        
//-----------------------------------------------------------------------------
// Name: reshapeFunc( )
// Desc: called when window size changes
//-----------------------------------------------------------------------------
void Game::reshapeFunc( const SDL_ResizeEvent & resize_e )
{                                
    int w = resize_e.w, h = resize_e.h;
    // save the new window size
    _window_width = w; _window_height = h;
    // map the view port to the client area
    glViewport( 0, 0, w, h );

    SDL_SetVideoMode(_window_width, _window_height,
				    0, SDL_SWSURFACE | SDL_RESIZABLE | SDL_OPENGL);

    do_gl_projection();
}

// *** Event Callbacks *** //

int Game::keydown_event(const SDL_KeyboardEvent & keyboard_e) {
    int key = keyboard_e.keysym.sym;
    switch (key) {
    case SDLK_ESCAPE:
    case SDLK_q:
        return SDL_QUIT;
    case SDLK_UP:
    case UP_KEY:
        _death_ball->_moving_forward = true;
        _death_ball->_moving_backward = false;
        break;
    case SDLK_LEFT:
    case LEFT_KEY:
        _death_ball->_moving_left = true;
        _death_ball->_moving_right = false;
        break;
    case SDLK_RIGHT:
    case RIGHT_KEY:
        _death_ball->_moving_left = false;
        _death_ball->_moving_right = true;
        break;
    case SDLK_DOWN:
    case DOWN_KEY:
        _death_ball->_moving_forward = false;
        _death_ball->_moving_backward = true;
        break;
    }
    return 0;
}

int Game::keyup_event(const SDL_KeyboardEvent & keyboard_e) {
    int key = keyboard_e.keysym.sym;
    switch (key) {
    case SDLK_UP:
    case UP_KEY:
        _death_ball->_moving_forward = false;
        break;
    case SDLK_LEFT:
    case LEFT_KEY:
        _death_ball->_moving_left = false;
        break;
    case SDLK_RIGHT:
    case RIGHT_KEY:
        _death_ball->_moving_right = false;
        break;
    case SDLK_DOWN:
    case DOWN_KEY:
        _death_ball->_moving_backward = false;
        break;
    }
    return 0;
}

GravityBall* Game::get_closest_ball() {
    double min_dist = -1;
    GravityBall* closest = NULL;
    for (unsigned i = 0; i < _gravity_balls.size(); i++) {
        if (_gravity_balls[i]->_is_exploding) continue;

        STVector3 d = _gravity_balls[i]->_pos - _death_ball->_pos;
        double dist = d.Length();
        if (dist > 10) continue;
        d /= dist;
        double dot = STVector3::Dot(d, _death_ball->_dir);
        if (dot < .8) continue;
        dist *= .1 + (1 - dot);

        if (dist < min_dist || min_dist == -1) {
            closest = _gravity_balls[i];
            min_dist = dist;
        }
    }
    return closest;
}

void Game::mouse_down_event(const SDL_MouseButtonEvent & button_e)
{
    //cout << "button pressed" << endl;
    switch (button_e.button) {
    case 1:
        //_death_ball->kill();
        _death_ball->hurt();
        break;
    case 2:
        //_gravity_balls[0]->crack();
//      _is_mouse_pressed = true;
//      for(unsigned k = 0; k < _buffer_fishes.size(); k++) {
//          BufferFish* fish = _buffer_fishes[k];
//          fish->_audio_loop->glitch();
//          fish->_audio_loop->stutter();
//      }
//      for(unsigned k = 0; k < _gravity_balls.size(); k++) {
//          GravityBall* gball = _gravity_balls[k];
//          gball->_audio_loop->glitch();
//          gball->_audio_loop->stutter();
//      }
        break;
    case 3:
        break;
//      {
//      AudioLoop* loop = new AudioLoop(_sound_sys);
//      loop->load_file("content/audio/loops/Deep House Dance Beat 04.wav");
//      GravityBall* gball = new GravityBall(loop);
//      _gravity_balls.push_back(gball);
//      loop->play();
//      gball->_pos = STVector3(_camera->_pos.x, _camera->_pos.y, _camera->_pos.z);
//      }
        break;
    case 4:
        //scroll wheel up
        break;
    case 5:
        //scroll wheel down
        break;
    }
}

void Game::mouse_up_event(const SDL_MouseButtonEvent & button_e)
{
    if (button_e.button != 1) return;
    _is_mouse_pressed = false;
}

void Game::mouse_motion_event(const SDL_MouseMotionEvent & motion_e)
{
    int dx = motion_e.x - _window_width/2;
    int dy = motion_e.y - _window_height/2;
    if (dx == 0 && dy == 0) return;

    _death_ball->change_yaw(dx);
    _death_ball->change_pitch(dy);

    if (_is_mouse_pressed) {
        // stub
    }
    SDL_WarpMouse(_window_width/2, _window_height/2);
}

void draw_floor() {

     GLfloat material_Ka[] = {0.0f, 0.2f, 0.0f, 1.0f};
     GLfloat material_Kd[] = {0.0f, 1.0f, 0.0f, 1.0f};
     GLfloat material_Ks[] = {1.0f, 1.0f, 1.0f, 1.0f};
     ALPHA_HACK(.5);
     GLfloat material_Se = 200.0f;

     glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, material_Ka);
     glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, material_Kd);
     glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, material_Ks);
     glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, material_Se);

    glBegin(GL_LINES);
    glColor4f(0,1,.3,.4);
    for(int j = -400; j < 400; j++) {
        glVertex3f(j, -1, -400);
        glVertex3f(j, -1, 400);
        glVertex3f(-400, -1, j);
        glVertex3f(400, -1, j);
    }
    glEnd();
}

//-----------------------------------------------------------------------------
// Name: render( )
// Desc: callback function invoked to draw the client area
//-----------------------------------------------------------------------------
void Game::render(double tick_time)
{
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    glNormal3f(0, 1, 0);
    draw_floor();

    ALPHA_HACK(.4);
    _death_ball->render(tick_time);
    ALPHA_HACK(1);

    for (unsigned j = 0; j < _gravity_balls.size(); j++) {
        _gravity_balls[j]->render(tick_time);
    }

    for (unsigned k = 0; k < _buffer_fishes.size(); k++) {
        _buffer_fishes[k]->render(tick_time);
    }

    //draw_death_ball();
//  glColor4f(1, .3, 0, .4);
//  STShape* sphere = STShapes::CreateSphere(.5, STPoint3(0,0,11), 16, 16);
//  sphere->Draw();
//  delete sphere;
//  sphere = STShapes::CreateSphere(.5, STPoint3(4,0,11), 16, 16);
//  sphere->Draw();
//  delete sphere;

    SDL_GL_SwapBuffers( );
}

void Game::garbage_collect() {

    {
        vector<BufferFish*>::iterator iter = _buffer_fishes.begin();
        while (iter != _buffer_fishes.end()) {

            BufferFish* fish = *iter;

            if (fish->_is_dead) {
                iter = _buffer_fishes.erase(iter);
                delete fish;
            } else {
                ++iter;
            }

        }
    }

    {
        vector<GravityBall*>::iterator iter = _gravity_balls.begin();
        while (iter != _gravity_balls.end()) {

            GravityBall* gball = *iter;

            if (gball->_is_dead) {
                if (_death_ball->_selected == gball) {
                    _death_ball->_selected = NULL;
                }
                iter = _gravity_balls.erase(iter);

                // remove any ties to fish
                for (unsigned k = 0; k < _buffer_fishes.size(); k++) {
                    if (_buffer_fishes[k]->_gravity_ball == gball) {
                        _buffer_fishes[k]->_gravity_ball = NULL;
                    }
                }

                delete gball;
            } else {
                ++iter;
            }

        }
    }
}

void Game::update(double tick_time)
{
    garbage_collect();

    for (unsigned j = 0; j < _gravity_balls.size(); j++) {
        _gravity_balls[j]->update(tick_time);
    }

    for(unsigned k = 0; k < _buffer_fishes.size(); k++) {
        BufferFish* fish = _buffer_fishes[k];
        fish->update(tick_time);
    }

    _death_ball->update(tick_time);
    _death_ball->select(get_closest_ball());

    _camera->update(tick_time);
    _sound_sys->update();

}


void Game::draw_death_ball()
{
    glEnable(GL_POINT_SMOOTH);
    glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);
    glPointSize(2);
    double size_mult = 2;

    glColor4f( 1,1,1, .4);
    for (int j = 3; j > 0; j--) {
        glLineWidth( j );
        glBegin( GL_POINTS );
        for( unsigned int i = 0; i < _buffer_size/2; i++ )
        {
            glVertex2f( size_mult * _audio_buffer[i], size_mult * _audio_buffer[i+ _buffer_size/2]);
        }
        for( unsigned int i = 0; i < _buffer_size/2; i++ )
        {
            glVertex2f( size_mult * _audio_buffer[i + _buffer_size/2], size_mult * _audio_buffer[i]);
        }
        glEnd();
    }
    for (int j = 3; j > 0; j--) {
        glLineWidth( j );
        glBegin( GL_LINE_STRIP );
        for( unsigned int i = 0; i < _buffer_size/2; i++ )
        {
            glVertex2f( size_mult * _audio_buffer[i], size_mult * _audio_buffer[i+ _buffer_size/2]);
        }
        for( unsigned int i = 0; i < _buffer_size/2; i++ )
        {
            glVertex2f( size_mult * _audio_buffer[i + _buffer_size/2], size_mult * _audio_buffer[i]);
        }
        glEnd();
    }
}

int Game::main_loop()
{

    bool quit = false;
    SDL_Event event;
    TickTimer tick_timer;
    TickTimer fps_tt;
    while (!quit) {
        if (SDL_PollEvent(&event)) {
            switch (event.type) {
            case SDL_MOUSEBUTTONDOWN:
                mouse_down_event(event.button);
                break;
            case SDL_MOUSEBUTTONUP:
                mouse_up_event(event.button);
                break;
            case SDL_MOUSEMOTION:
                mouse_motion_event(event.motion);
                break ;
            case SDL_VIDEORESIZE:
                reshapeFunc(event.resize);
                break;
            case SDL_KEYDOWN:
                if (keydown_event(event.key) == SDL_QUIT) quit = true;
                break;
            case SDL_KEYUP: 
                keyup_event(event.key);
                break;
            case SDL_QUIT:    
                quit = true;
                break;
            }
        } else {
            fps_tt.tick();
            double time_elapsed = tick_timer.tick();
            update(time_elapsed);
            render(time_elapsed);

            double sleep_time = 1.0/TARGET_FPS - fps_tt.tick();
            if (sleep_time > 0)
                usleep(sleep_time * 1000000);
            //cout << "fps: " << 1/time_elapsed << endl;
        }
    }
    clean_up();
    return 0;
}

